#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdCategory < ActiveRecord::Base
  belongs_to :user
  has_many :sidfmd_categories_hosts
  has_and_belongs_to_many :sidfmd_hosts
  has_and_belongs_to_many :users
  has_many :sidfmd_cat_groups_categories
  has_and_belongs_to_many :sidfmd_cat_groups
  has_many :sidfmd_rules, dependent: :destroy
  has_many :sidfmd_rule_t_vulsyncs, dependent: :destroy
  has_many :sidfmd_rule_a_mail_categories, dependent: :destroy
  has_many :sidfmd_rule_cm_category_subs, dependent: :destroy
  has_many :sidfmd_regkey_cats, dependent: :destroy
  has_many :sidfmd_host_timelines, dependent: :destroy
  has_many :sidfmd_vul_timelines, dependent: :destroy
  has_many :sidfmd_ctags, dependent: :destroy
  has_many :sidfmd_grp_filters, dependent: :destroy

  scope :filter_user, -> { where('user_id IS NULL OR user_id = ?', User.anonymous? ? User.current : nil) }
  scope :system, -> { select(:id, :name, :desc).where(:is_system => true).order(:id) }
  scope :not_system, -> { select(:id, :name, :desc, :user_id).where(:is_system => false).order(:name, :id) }
  scope :with_hosts, -> { joins(:sidfmd_categories_hosts).distinct }
  scope :with_vuls, -> { where(:id => SidfmdCategoriesHost.enable_hosts.where(:sidfmd_host_id => SidfmdIssue.enable_hosts.select(:sidfmd_host_id).distinct).select(:sidfmd_category_id).distinct) }

  before_save :strip_name
  #validates :hostlim, numericality: { greater_than_or_equal_to: 0, :allow_blank => true }, if: -> { respond_to?(:hostlim) }

  # カスタムカテゴリのバリデーション
  # エラー内容は、下記のようにして、取得する。 下記の例ではカテゴリ名のエラー
  # category.save または category.valid? がfalseなら、category.errors[:name] で取得する
  validate do |category|
    unless category.is_system then
      name = category.name
      if name.nil? or name.size == 0 then
        category.errors[:name] << 'カテゴリ名が入力されていません。'
      else
        category.strip_name
        name = category.name
        if SidfmdCategory.where(:name => name).where.not(:id => category.id ? category.id : -1).count > 0 then
          category.errors[:name] << '同名のカテゴリが存在します。'
        elsif name.size > 250 then
          category.errors[:name] << 'カテゴリ名が長すぎます。（最大250文字）'
        end
      end
      desc = category.desc
      if desc.nil? or desc.size == 0 then
        category.errors[:desc] << '概要が入力されていません。'
      elsif desc.size > 1000 then
        category.errors[:desc] << '概要が長すぎます。（最大1000文字）'
      end
      if category.respond_to?(:hostlim) && category.hostlim then
        if category.hostlim < 0 then
          category.errors[:hostlim] << "#{HOSTLIM_LABEL}に、負の値は設定できません。"
        elsif !SidfmdCategory.hostlim_valid?(category.hostlim) then
          category.errors[:hostlim] << "#{HOSTLIM_LABEL}に設定可能な値は、最大#{SidfmdCategory.max_hostlim}(ライセンスのホスト数上限値)です。"
        end
      end
    end 
  end

  #カテゴリ名の最大文字数
  MAX_NAME_LENGTH = 250
  #概要の最大文字数
  MAX_DESC_LENGTH = 1000

  #hostlimのラベル
  HOSTLIM_LABEL = 'ホスト登録上限数'

  # システムカテゴリのラベル
  PublicServer = :public_server
  PrivateServer = :private_server
  SharedServer = :shared_server
  Mobile = :mobile
  Client = :client
  VirtualHostServer = :virtual_host_server
  PublicNetwork = :public_network
  PrivateNetwork = :private_network
  LABELS = [ PublicServer, PrivateServer, Mobile, Client, SharedServer, VirtualHostServer,
             PublicNetwork, PrivateNetwork ].freeze
  NAMES = [ '公開サーバ', '内部サーバ', 'モバイル', 'クライアント', '共用サーバ', '仮想化ホスト', '公開NW機器', '内部NW機器' ].map(&:freeze).freeze

  def self.label2id(label)
    idx = LABELS.index(label)
    idx ? idx + 1 : 0
  end

  def self.id2label(id)
    if id >= 1 and id <= 8 then
      LABELS[id - 1]
    else
      nil
    end
  end

  def strip_name
    self.name = self.name.strip
  end

  # 属するホストがあるならtrue
  def has_host?
    SidfmdCategoriesHost.where(:sidfmd_category_id => id).where(:sidfmd_host_id => SidfmdHost.enable_hosts.select(:id)).count > 0
  end

  # htmlidに関連するsidfmd_issuesを抽出し、issuesのsidfmd_host_id
  # hostidに関連するカテゴリ一覧を取得し返す。
  def self.get_categories_htmlid(htmlid)
    self.joins(:sidfmd_categories_hosts).joins(:sidfmd_categories_hosts => [ :sidfmd_host ]).joins(:sidfmd_hosts => [ :sidfmd_issues ]).where(:sidfmd_issues => { :html_id => htmlid }).order(:id)
  end

  # ホスト登録上限数に登録できる最大値を返す
  def self.max_hostlim
    hosts = SidfmdHost.get_hosts_by_lm(:'sort[hostname]' => :asc)
    num = hosts ? hosts[:license_number] : nil
  end

  # 指定したホスト登録上限数が有効かどうかを返す
  def self.hostlim_valid?(hostlim)
    ret = true
    if hostlim then
      max_host_num = self.max_hostlim
      if max_host_num then
        ret = hostlim.to_i <= max_host_num.to_i
      end
    end
    ret
  end

  # カテゴリ関連のViewを更新
  def self.update_view(html_ids:, host_ids:)
    SidfmdVulView.sync_data(html_ids.uniq) if html_ids.any?
    SidfmdHostView.sync_data(host_ids.uniq) if host_ids.any?
  end

  # カテゴリ関連のルールを処理
  def self.process_related_rules(**opts)
    r = true
    param = {
      :role_admin => opts[:sidfm_roles].include?(:admin),
      :user_category_ids => (opts[:sidfm_roles].include?(:admin) ? nil : User.current.categories.pluck(:id)),
      :action => opts[:action],
      :messages_necessary => false
    }
    case opts[:action]
    when 'delete' then
      param[:delete_category_ids] = opts[:delete_category_ids]
    when 'import' then
      param[:delete_category_ids] = opts[:delete_category_ids]
    else
      #invalid
    end
    ret = SidfmdRule.categories_related_rule_info(param)
    SidfmdRuleCmCategorySub.where(:id => ret[:delete_sub_category_cond_ids]).destroy_all if ret[:delete_sub_category_cond_ids].any?
    SidfmdRuleCmHostSub.where(:id => ret[:delete_sub_host_cond_ids]).destroy_all if ret[:delete_sub_host_cond_ids].any?
    SidfmdRuleChpUserSub.where(:id => ret[:delete_chp_user_sub_ids]).destroy_all if ret[:delete_chp_user_sub_ids].any?
    SidfmdRuleCmCtag.where(:id => ret[:delete_cm_ctag_ids]).destroy_all if ret[:delete_cm_ctag_ids].any?
    SidfmdRuleAmCtag.where(:id => ret[:delete_am_ctag_ids]).destroy_all if ret[:delete_am_ctag_ids].any?
    SidfmdRuleCond.where(:id => ret[:delete_cond_ids]).destroy_all if ret[:delete_cond_ids].any?
    SidfmdRuleAction.where(:id => ret[:delete_action_ids]).destroy_all if ret[:delete_action_ids].any?
    SidfmdRule.where(:id => ret[:delete_rule_ids]).destroy_all if ret[:delete_rule_ids].any?
    r
  end

  # エラーメッセージを生成
  def self.create_err_message(errors)
    messages = []
    messages<< errors[:name] if errors[:name]
    messages<< errors[:desc] if errors[:desc]
    messages<< errors[:hostlim] if errors[:hostlim]
    messages.join
  end

end
