#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmCategoriesControllerSupport
  include SidfmVmApiControllerSupport
  # 既存のメソッドとメソッド名が被らないように注意

  def get_openid_connect_category_id
    category_id = -1
    #sidfm_categoriesは今のところ1つのみ設定可能になっている
    category_ids = Sidfm.openid_connect_settings_user_category_ids
    category_id = category_ids.split(',').first.to_i if category_ids.length > 0
    category_id
  end

  def openid_connect_enable?
    settings = (Setting.send('plugin_redmine_openid_connect')).symbolize_keys
    ret = (settings[:enabled] ? true : false) #enabledがある時は、文字列のfalseになっている
  end

  # 新規登録
  def create_category(name:, desc:, hostlim:)
    begin
      category = SidfmdCategory.new(
        :name => name,
        :desc => desc,
        :hostlim => hostlim,
        :is_system => false
      )
   
      unless category.save then
        message = SidfmdCategory.create_err_message category.errors
        return create_process_result message
      end
    rescue Exception => e
      log_exception(e)
      return create_process_result 'カテゴリの作成に失敗しました。'
    end
    return create_process_result nil, { :id => category.id }
  end

  # カテゴリ更新
  def update_category(id:, name:, desc:, hostlim:, update_view:)
    host_ids = []
    html_ids = []
    begin
      SidfmdCategory.transaction do
        category = SidfmdCategory.lock.find_by_id(id.to_i)
        unless category then
          return create_process_result '更新対象のカテゴリが存在しません。'
        end
        if category && !category.is_system then
          if category.update_attributes(
              :name => name,
              :desc => desc,
              :hostlim => hostlim,
            )
          then
            #View更新のためのid取得
            host_ids =
              SidfmdHostView.where(
                :id => SidfmdHvCategory.where(:sidfmd_category_id => id.to_i)
                                       .select(:sidfmd_host_view_id)
              )
              .pluck(:sidfmd_host_id)
            html_ids = SidfmdHostView.relate_htmlids(host_ids).uniq
          else
            message = SidfmdCategory.create_err_message category.errors
            return create_process_result message
          end
        end
      end
    rescue Exception => e
      log_exception(e)
      return create_process_result 'カテゴリの更新に失敗しました。'
    end
    Rails.cache.clear

    if update_view then
      # API等、すぐに更新が必要な場合にViewを更新
      begin
        SidfmdCategory.update_view(:html_ids => html_ids, :host_ids => host_ids)
      rescue Exception => e
        logger.info('failed to update data related to view')
        log_exception(e)
      end
    end

    return create_process_result nil, { :html_ids => html_ids, :host_ids => host_ids }
  end

  # カテゴリ削除
  # エラーハンドリング等は呼び出し元で行う
  def delete_category(id:, openid_connect_auth_name:, update_view:, sidfm_roles:)
    id = id.to_i
    if openid_connect_enable? and id == get_openid_connect_category_id then
      return create_process_result "#{openid_connect_auth_name}のカスタムカテゴリに指定されているため、削除できません。"
    end
    ctag_related_issue_ids = []
    begin
      SidfmdCategory.transaction do
        category = SidfmdCategory.lock.find_by(:id => id)
        if category and !category.is_system then
          #関連するデータの削除
          process_rules = SidfmdCategory.process_related_rules(
            :sidfm_roles => sidfm_roles,
            :delete_category_ids => [ id ],
            :action => 'delete'
          )
          if !process_rules then
            raise Exception
          end
          SidfmdCategoriesHost.where(:sidfmd_category_id => id).delete_all
          SidfmdCategoriesUser.where(:sidfmd_category_id => id).delete_all
          cat_ctags_ids = SidfmdCtag.where(:sidfmd_category_id => id).pluck(:id)
          ctag_related_issue_ids = SidfmdCtag.delete_ctags(cat_ctags_ids)
          cat_groups = SidfmdCatGroup.only_categories([id])
          if cat_groups.size > 0 then
            SidfmdCatGroup.where(:id => cat_groups.map{|grp| grp[:id]}).destroy_all
          end
          category.destroy # カテゴリ削除
        else
          return create_process_result "削除に失敗しました。"
        end
      end
    rescue Exception => e
      log_exception(e)
      return create_process_result 'カテゴリの削除に失敗しました。'
    end
    Rails.cache.clear
    host_ids = SidfmdHostView.where(
      :id => SidfmdHvCategory.where(:sidfmd_category_id => id).select(:sidfmd_host_view_id)
    ).pluck(:sidfmd_host_id)
    html_ids = SidfmdHostView.relate_htmlids(host_ids).uniq
    html_ids |= SidfmdIssue.where(:id => ctag_related_issue_ids).distinct.pluck(:html_id) if ctag_related_issue_ids.any?
    
    if update_view then
      # API等、すぐに更新が必要な場合にViewを更新
      begin
        SidfmdCategory.update_view(:html_ids => html_ids, :host_ids => host_ids)
      rescue Exception => e
        logger.info('failed to update data related to view')
        log_exception(e)
      end
    end
    return create_process_result nil, { :html_ids => html_ids, :host_ids => host_ids }
  end

end
