#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class CreateSidfmdLocalVuls < ActiveRecord::Migration
  def change
    # ローカル脆弱性情報
    create_table :sidfmd_local_vuls do |t|
      # 有効・無効。無効なら表示しない。
      t.boolean :flg_enable, :null => false, :default => true, :index => true

      # html_id。SIDfmIssue#html_idに格納する脆弱性ID。
      # modelで自動生成するので、createの際には設定不要。
      t.integer :html_id, :null => false, :index => true

      # 脆弱性タイトル
      t.text :title, :null => false

      # 危険度
      # SidfmdRisk#risk_idの値を設定。
      # 内容は以下で確認
      #  SidmfdRisk.all
      # (注意)SidfmdRisk#idを設定しないこと。
      t.integer :risk_id, :null => false, :index => true

      # 影響範囲
      # SidfmdArea#area_idの値を設定
      # 内容は以下で確認
      #  SidfmdArea.all
      # (注意)SidfmdArea#idを設定しないこと。
      t.integer :area_id, :null => false, :index => true

      # 簡易説明
      # 概要を要点のみにして短くしたもの。CSVやJSONデータに出力される。
      # 入力が無ければ、脆弱性タイトルで埋めておく。
      t.text :short_description, :null => false

      # 以下は脆弱性の内容のテキストを格納する。
      # 保存形式は通常のテキストデータで、HTMLに出力する際に以下の変換を
      # 行う。
      # - 改行コードを<br />に変換
      # - リンク用タグの変換処理
      #   以下のような変換を行う。
      #
      #   [元テキストデータ]
      #   %LINK:ソフテック:http://www.softek.co.jp/%
      #   [変換後]
      #   <a href="http://www.softek.co.jp/" target="_blank">ソフテック</a>
      #
      #   %LINK:<リンクタイトル>:<URL>% のようなフォーマットになっている。
      #   URLについては、URI.parseでのチェック必須。URI#schemeはhttp,
      #   https, ftp以外は不許可。
      #   リンクタイトル、URLについては、<>"\&は使用不可。
      #   なお、似たような機能のsimple_formatがRailsにあるが、利用しないこと。
      #   セキュリティ上の問題（主にクロスサイトスクリプティング）に繋がりや
      #   すいので、UI部分については十分に注意して実装すること。

      # 概要
      t.text :description, :null => false
      # 影響などの確認方法
      t.text :confirmation, :null => false
      # 対処方法
      t.text :workaround, :null => false
      # 関連情報
      t.text :related, :null => false

      t.timestamps
    end

    # 更新履歴
    # 新規登録時も「2020/07/28 コンテンツ 登録」のような記録を作成する。
    # その後ローカル脆弱性を更新する際には、更新履歴のメッセージも入力
    # 項目として用意し、更新履歴に追記ができるようにしておく。
    create_table :sidfmd_local_vul_logs do |t|
      t.references :sidfmd_local_vul, :null => false, :index => true
      t.date :dt, :null => false
      t.text :mes, :null => false
    end

    # 影響を受ける製品(SIDfm)
    # pid=SidfmdProduct#pid
    # vid=SidfmdVersion#vid
    # この情報を元にホストと関連付けを実施。
    create_table :sidfmd_local_vul_apps do |t|
      t.references :sidfmd_local_vul, :null => false, :index => true
      t.integer :pid, :null => false, :index => true
      t.integer :vid, :null => true, :index => true
    end

    # 影響を受ける製品(ローカル)
    # この情報を元にホストと関連付けを実施。
    create_table :sidfmd_local_vul_lapps do |t|
      t.references :sidfmd_local_vul, :null => false, :index => true

      t.references :sidfmd_lproduct, :null => false, :index => true
      t.references :sidfmd_lversion, :null => true, :index => true
    end

    # ローカル脆弱性情報のCVE ID
    # 複数登録可。表示されるのは、基本的にbase_scoreが最大のもの1つのみ。
    # 登録されていない場合は cvss not_reg になる。
    # SidfmdIssue登録の際にSidfmdIssueCveに登録するデータです。
    create_table :sidfmd_local_vul_cves do |t|
      t.references :sidfmd_local_vul, :null => false, :index => true

      # CVE ID
      # ex. CVE-2015-9541
      t.text :cve_id, :null => false

      # CVSSv2値
      # 設定無しならnull
      t.float :c2_base_score, :index => true

      # CVSSv2の各要素を示す文字列
      # どんな文字列が入るかは以下を参照。CVSS2の場合はc2_vector。
      #  SidfmdIssueCve.where.not(:c3_vector => nil).last.c3_vector
      # 値をどのように利用するかは/sidfm_information/vulnの処理を参照。
      t.text :c2_vector

      # 攻撃コードの有無(CVSSv2)
      t.boolean :c2_exploitable, :null => false, :default => false, :index => true

      # 以下はCVSSv3
      t.float :c3_base_score, :index => true
      t.text :c3_vector
      t.boolean :c3_exploitable, :null => false, :default => false, :index => true

      t.timestamps
    end

    # ローカル脆弱性情報のSRI情報
    # 一つのsidfmd_local_vulに対して、8個(category_id=1～8)登録する必要がある。
    # SidfmdIssue登録時にSidfmdIssueSriに登録するデータです。
    create_table :sidfmd_local_vul_sris do |t|
      t.references :sidfmd_local_vul, :null => false, :index => true

      # システムカテゴリのID(1-8:SidfmdCategory.system.map(&:id))
      t.integer :category_id, :null => false

      # システムカテゴリに対応する危険度(0-4:SidfmdIssue::VALUES)
      t.integer :risk_id, :null => false

      # 補足情報
      # 記載例は以下を参照。
      #   SidfmdIssueSri.where.not(:memo => nil).map(&:memo)
      t.text :memo

      t.timestamps
    end
  end
end
