#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

require 'pp'
require 'time'
require 'digest'
require 'fileutils'

require 'vm_rule'

module SIDfm
  class VMDaemon

    # 各種操作に使用するアカウント。基本的には変更しないこと。
    ADMIN_USER = User.find_by_login('sidfmadm')

    ISSUE_STATUS_FIXED = IssueStatus.find(3).name
    ISSUE_STATUS_NOTCONFIRMED = IssueStatus.find(1).name

    NOTE_NO_LONGER_DETECTED = "脆弱性情報との照合の結果、脆弱性が検出されなくなったため、ステータスを#{ISSUE_STATUS_FIXED}に変更しました。"

    NOTE_CHANGE_PRODVER = "対象の脆弱性について、影響のあるプロダクトの情報が更新されたため、ステータスを#{ISSUE_STATUS_NOTCONFIRMED}に変更しました。お手数ですが脆弱性の内容を再度ご確認の上、対応をお願いします。"

    NOTE_CHANGE_CVE = "対象の脆弱性について、CVE/CVSSが更新されたため、ステータスを#{ISSUE_STATUS_NOTCONFIRMED}に変更しました。お手数ですが脆弱性の内容を再度ご確認の上、対応をお願いします。"

    WIN_RISKS = {
      'Critical' => 4,
      'Important' => 3,
      'Moderate' => 2,
      'Low' => 1,
      nil => 4,
    }.freeze

    VM_TRACKER = Tracker.find_by_name('SIDfm脆弱性管理')
    VM_PROJECT = Project.find_by_name('脆弱性情報管理')

    DEFAULT_WIN_AREA = SidfmdArea.find_by_area_id(0)

    DEFAULT_PRIORITY = IssuePriority.find_by_position_name('default')

    LOGDIR = '/var/lib/redmine/log'
    LOG_INFO = 'vm_daemon-info.log'
    LOG_ERROR = 'vm_daemon-error.log'
    LOG_DEBUG = 'vm_daemon-debug.log'
    LOG_SIZE = 10 * 1024 * 1024

    DEBUG = FileTest.exist?('/flag.debug')

    private

    def initialize
      # vm_daemonの処理で新規登録・更新・脆弱性除外の情報を保持
      # sync_hostids: 同期対象ホストのid一覧
      # prev_vuls_content : 同期前の脆弱性一覧(必要なフィールドのみ)
      # add_htmlids: 登録されるsidfmd_issue_idに対応するhtml_idとsidfmd_hpst_idとの対応の配列
      # modify_htmlids: 変更されるsidfmd_issue_idに対応するhtml_idとsidfmd_host_idとの対応の配列
      # remove_htmlids: 影響無に変更されたhtmlids,sidfmd_host_idの対応の配列
      @rule_vulsync = {
        :sync_hostids => [],
        :prev_vuls_content => SidfmdVulView.select(:html_id).select(:exploitable).collect {|x| { :html_id => x.html_id, :exploitable => x.exploitable } },
        :add_htmlids => [],
        :modify_htmlids => [],
        :remove_htmlids => [],
      }
      # timelineで取得されるコンテンツを保存して、vm_ruleの処理で利用する
      @rule_contents = { :vulsync_content => {}, }
      @htmlid_date_htmlids = Set.new
    end

    def logging(file, *s)
      if FileTest.exist?(file) and File.size(file) > LOG_SIZE then
        8.times do |i|
          file_n = file + ".#{8 - i}"
          file_old = file + ".#{8 - i + 1}"
          if FileTest.exist?(file_n) then
            FileUtils.mv(file_n, file_old, :force => true)
          end
        end
        if FileTest.exist?(file) then
          FileUtils.mv(file, file + ".0", :force => true)
        end
      end
      File.open(file, 'a') do |f|
        s.each do |i|
          f.puts(s.kind_of?(String) ? i : i.inspect)
        end
      end
    end

    def log_info(*s)
      logging("#{LOGDIR}/#{LOG_INFO}", *s)
    end

    def log_error(*s)
      logging("#{LOGDIR}/#{LOG_ERROR}", *s)
    end

    def log_debug(*s)
      logging("#{LOGDIR}/#{LOG_DEBUG}", *s) if DEBUG
    end

    # 指定したsidfmd_issueについて、ステータスを未対策に変更する。
    def change_status_not_confirmed(issue)
      flg = Sidfm.vmdaemon_affect_product
      log_info(" vmdaemon_affect_product: #{flg}")
      if flg then
        @rule_vulsync[:modify_htmlids] << { :html_id => issue[:html_id], :sidfmd_host_id => issue[:sidfmd_host_id] }
        issue.change_status_with_journal(ADMIN_USER.id,
                                         SidfmdIssue::STATUS_NOTCONFIRMED,
                                         NOTE_CHANGE_PRODVER)
      end
    end

    # 指定したsidfmd_issueについて、ステータスを未対策に変更する。
    # CVE版。
    def change_status_not_confirmed_cve(issue)
      @rule_vulsync[:modify_htmlids] << { :html_id => issue[:html_id], :sidfmd_host_id => issue[:sidfmd_host_id] }
      issue.change_status_with_journal(ADMIN_USER.id,
                                       SidfmdIssue::STATUS_NOTCONFIRMED,
                                       NOTE_CHANGE_CVE)
    end

    # 指定したsidfmd_issueについて、ステータスを対策済に変更する。
    def change_status_fixed(issue)
      @rule_vulsync[:remove_htmlids] << { :html_id => issue[:html_id], :sidfmd_host_id => issue[:sidfmd_host_id] }
      issue.change_status_with_journal(ADMIN_USER.id,
                                       SidfmdIssue::STATUS_FIXED,
                                       NOTE_NO_LONGER_DETECTED)
    end

    public

    # 影響範囲の同期
    def areas_update
      areas = nil
      Sidfm.sidapi do |o|
        areas = o.areas
      end
      if areas and areas['areas'] then
        areas = areas['areas']
        areas.each do |area|
          areas_update_unit(area)
        end
      end
    end

    # areaの内容とSidfmdAreaの内容を照合して、
    # 必要があれば追加・更新。
    def areas_update_unit(area)
      SidfmdArea.transaction do
        local_area = SidfmdArea.find_by_area_id(area['id'])
        if local_area then
          if local_area.name != area['name'] then
            unless local_area.update_attribute(:name, area['name']) then
              raise ActiveRecord::Rollback
            end
          end
        else
          area['area_id'] = area['id']
          area.delete('id')
          local_area = SidfmdArea.create(area)
        end
      end
    end

    # 危険度の同期
    def risks_update
      risks = nil
      Sidfm.sidapi do |o|
        risks = o.risks
      end
      if risks and risks['risks'] then
        risks = risks['risks']
        risks.each do |risk|
          risks_update_unit(risk)
        end
      end
    end

    # riskの内容とSidfmdRiskの内容を照合して、
    # 必要があれば追加・更新
    def risks_update_unit(risk)
      SidfmdRisk.transaction do
        local_risk = SidfmdRisk.find_by_risk_id(risk['id'])
        if local_risk then
          if local_risk.name != risk['name'] then
            unless local_risk.update_attribute(:name, risk['name']) then
              raise ActiveRecord::Rollback
            end
          end
          if local_risk.name_en != risk['name_en'] then
            unless local_risk.update_attribute(:name_en, risk['name_en']) then
              raise ActiveRecord::Rollback
            end
          end
        else
          risk['risk_id'] = risk['id']
          risk.delete('id')
          local_risk = SidfmdRisk.create(risk)
        end
      end
    end

    # プロダクト、バージョン情報の更新
    def products_update(force = nil)
      data = nil
      now = Time.current
      Sidfm.sidapi do |o|
        begin_datetime = Sidfm.prodver_last_update ? Sidfm.prodver_last_update.to_datetime.to_s : nil
        end_datetime = now.to_datetime.to_s
        if force or begin_datetime.nil? then
          data = o.products2
        else
          data = o.products2('begin_datetime' => begin_datetime, 'end_datetime' => end_datetime)
        end
      end
      if data and data['products'] then
        products = data['products']
        Sidfm.transaction do
          products.each do |product|
            products_update_unit(product)
          end
          Sidfm.prodver_last_update = now
        end
      end
    end

    # productの内容とSidfmdProduct,SidfmdVersionの内容を照合して、
    # 必要があれば追加・更新 ※ 削除はここではできない
    def products_update_unit(product)
      ActiveRecord::Base.transaction(requires_new: true) do
        versions = product['versions']
        exist_product = SidfmdProduct.find_by_pid(product['id'])
        unless exist_product then
          product['pid'] = product['id']
          product.delete('id')
          product.delete('versions')
          exist_product = SidfmdProduct.create(product)
        end
        if versions then
          versions.each do |version|
            exist_version = SidfmdVersion.find_by_vid(version['id'])
            if exist_version then
              if exist_version['update_datetime'] and exist_version['update_datetime'] < version['update_datetime'] then
                version.delete('id')
                version.delete('vid')
                unless exist_version.update_attributes(version) then
                  raise ActiveRecord::Rollback
                end
              end
            else
              version['vid'] = version['id']
              version['sidfmd_product_id'] = exist_product.id
              version.delete('id')
              exist_version = SidfmdVersion.create(version)
            end
          end
        end
      end
    end

    # queueの情報からSIDAPIで脆弱性情報を取得し、保存する
    # 保存先：SidfmdQueue*
    # 終了後、queue.done が false -> true に変化。
    def issues_update(target_hosts)
      flg_do_cache_clear = false
      target_hosts.each do |host|
        queue = nil
        win_queue = nil
        log_debug(" start: #{host.hostname}")
        opts = {
          :product_ids => [],
          :product_version_ids => [],
          :packages => [],
          :begin_datetime => Time.at(0).to_datetime.to_s,
          :end_datetime => Time.current.to_datetime.to_s,
        }
        if host.refdate then
          begin
            opts[:begin_datetime] = DateTime.parse(host.refdate.to_s + ' 00:00:00 +09:00:00').to_s
          rescue Exception => e
            log_error(e.message)
            if e.backtrace then
              e.backtrace do |i|
                log_errror(i)
              end
            end
          end
        end
        if host.is_agent then
          log_debug(" agent exist")
          # agent 有り
          queue = host.get_last_queue
          if queue then
            log_debug(" queue exist")
            # queueあり
            win_queue = queue.sidfmd_queue_win
            issues_update_with_queue(host, queue, opts)
          else
            log_debug(" not queue exist")
            # queue無し
            issues_update_without_queue(host, opts)
          end
        else
          log_debug(" no agent exist")
          # agent無し
          issues_update_without_agent(host, opts)
          # agent無しの場合のqueueはここで生成する。
          queue = SidfmdQueue.create(:sidfmd_host_id => host.id,
                                     :done => false,
                                     :checked => false)
        end
        if win_queue then
          # AgentありのWindows用処理
          flg_do_cache_clear = issues_update_win(host, queue, win_queue)
        end
        if queue then
          issues_update_main(host, queue, opts)
          flg_do_cache_clear = true
        end
        ActiveRecord::Base.transaction do
          host.update_attribute :last_checked_at, Time.current
          host.update_attribute :last_checked_hash, calc_hash(opts)
        end
        log_debug(" end: #{host.hostname}")
      end
      if flg_do_cache_clear then
        Rails.cache.clear
      end
    end

    # windows agent用 issues_update
    # ここではAgentから受信したデータ(html_id < 0)のみを取り扱う。
    def issues_update_win(host, queue, win_queue)
      # html_id < 0 のSidfmdIssueのみを対象とした処理。
      # 登録済のissueとqueueのデータとの比較
      now_kbs = host.sidfmd_issues
        .where('sidfmd_issues.html_id < 0')
        .select(:html_id)
        .collect {|x| x.html_id }.uniq
      new_kbs = win_queue.sidfmd_queue_win_details.collect {|x| x.kb ? -x.kb : nil }.compact.uniq
      append_kbs = new_kbs - now_kbs
      removed_kbs = now_kbs - new_kbs
      SidfmdIssue.transaction do
        # issue追加
        append_kbs.each do |kb|
          # 既に登録されているかどうか確認
          exist_issue = host.sidfmd_issues.where(:html_id => kb)
          if exist_issue.count > 0 then
            exist_issue = exist_issue.first
            # 登録されていればステータス確認し、済・無なら未対策へ変更
            o_issue = exist_issue.issue
            if SidfmdIssue::STATUS_FINISHED.include?(o_issue.status_id) then
              change_status_not_confirmed(exist_issue)
            end
          else
            # 登録されていなければ新規作成。
            wqdetails = win_queue.sidfmd_queue_win_details.where(:kb => -kb)
            if wqdetails.count > 0 then
              wqdetail = wqdetails.order(:updated_at => :desc).limit(1).first
              new_issue_opts = {
                :tracker_id => VM_TRACKER.id,
                :project_id => VM_PROJECT.id,
                :subject => "[#{host.hostname}]" + wqdetail.title,
                :description => "新規脆弱性登録",
                :author_id => ADMIN_USER.id,
                :priority_id => DEFAULT_PRIORITY.id,
                :status_id => SidfmdIssue::STATUS_NOTCONFIRMED,
              }
              new_issue = Issue.create(new_issue_opts)
              if new_issue then
                new_sidfmd_issue_opts = {
                  :issue_id => new_issue.id,
                  :sidfmd_host_id => host.id,
                  :html_id => kb,
                  :title => wqdetail.title,
                  :risk_id => WIN_RISKS[wqdetail.severity],
                  :area_id => DEFAULT_WIN_AREA.area_id,
                }
                @rule_vulsync[:add_htmlids] << { :html_id => kb, :sidfmd_host_id => host.id }
                new_sidfmd_issue = SidfmdIssue.create(new_sidfmd_issue_opts)
                unless new_sidfmd_issue then
                  logger.error("cannot create sidfmd_issue: SidfmdIssue.create(#{new_sidfmd_issue_opts.inspect})")
                end
              else
                logger.error("cannot create issue: Issue.create(#{new_issue_opts.inspect})")
              end
            else
              logger.error("cannot create issue: lack queue_win_details.where(:kb => -#{kb})")
            end
          end
        end
        # 対策済への変更
        removed_kbs.each do |kb|
          # issueを取得
          exist_issue = host.sidfmd_issues.where(:html_id => kb)
          if exist_issue.count > 0 then
            exist_issue = exist_issue.order(:id => :desc).limit(1)
            if exist_issue then
              exist_issue = exist_issue.first
              # ステータス確認し、未・中なら対策済へ変更
              o_issue = exist_issue.issue
              if SidfmdIssue::STATUS_UNFINISHED.include?(o_issue.status_id) then
                change_status_fixed(exist_issue)
              end
            else
              logger.error("cannot get exist issue: exist_isue.order(:id => :desc).limit(1))")
            end
          else
            logger.error("cannot get exist issue: host(#{host.id}).sidfmd_issues.where(:html_id => #{kb})")
          end
        end
      end
      ret = (append_kbs.size > 0 or removed_kbs.size > 0)
      ret
    end

    # issues_update agent有りqueue有り
    def issues_update_with_queue(host, queue, opts)
      check_dist_pkgs(queue, opts)
      check_applications(queue, opts)
      issues_update_without_agent(host, opts)
      if host.refdate then
        begin
          opts[:begin_datetime] = DateTime.parse(host.refdate.to_s + ' 00:00:00 +09:00:00').to_s
        rescue Exception => e
          log_error(e.message)
          if e.backtrace then
            e.backtrace.each do |i|
              log_error(i)
            end
          end
        end
      else
        # epoch
        opts[:begin_datetime] = Time.at(0).to_datetime.to_s
      end
    end

    # issues_update agent有りqueue無し
    def issues_update_without_queue(host, opts)
      # no-op
    end

    # issues_update agent無し
    def issues_update_without_agent(host, opts)
      apps = host.sidfmd_host_apps
      if apps.count > 0 then
        apps.each do |app|
          if app.version_id and app.version_id > 0 then
            opts[:product_version_ids] << app.version_id
          elsif app.product_id and app.product_id > 0 then
            opts[:product_ids] << app.product_id
          end
        end
      end
    end

    # product_ids, product_version_ids, packages でHash値を生成する。
    def calc_hash(opts)
      pids = opts[:product_ids]
      vids = opts[:product_version_ids]
      pkgs = opts[:packages]
      pids = [] unless pids
      vids = [] unless vids
      pkgs = [] unless pkgs
      pids = pids.uniq.sort.join(',')
      vids = vids.uniq.sort.join(',')
      pkgs = pkgs.uniq.sort.join(',')
      [ Digest::SHA256.hexdigest(pids), Digest::SHA256.hexdigest(vids), Digest::SHA256.hexdigest(pkgs) ].join('-')
    end

    # optsの情報をSIDAPIで照合して脆弱性を取得し、SidfmdQueue*に記録
    def issues_update_main(host, queue, opts)
      log_debug(" issues_update_main")
      vuls = { 'vulns' => [] }
      new_hash = calc_hash(opts)
      old_hash =  host.last_checked_hash
      flag_diff = (old_hash != new_hash)
      ### FIXME: 差分更新は強制停止中
      flag_diff = true
      if !flag_diff then
        last_checked_at = host.last_checked_at
        if last_checked_at then
          opts[:begin_datetime] = last_checked_at.to_datetime.to_s
        end
      end
      if opts[:product_ids].size > 0 or
          opts[:product_version_ids].size > 0 or
          opts[:packages].size > 0 then
        Sidfm.sidapi(host.hostname, host.host_apikey) do |o|
          vuls = o.products_vulns(product_ids: opts[:product_ids].uniq,
                                  product_version_ids: opts[:product_version_ids].uniq,
                                  packages: opts[:packages],
                                  begin_datetime: opts[:begin_datetime],
                                  end_datetime: opts[:end_datetime],
                                  time_field: 'affect_product',
                                  package_scan_period: 'all')
        end
      end
      if vuls and vuls['vulns'] then
        log_debug(" process api response")
        vuls = vuls['vulns']
        log_debug("  vuls_size = #{vuls.size}")
        # update sidfmd_htmlid_dates
        vuls.each do |vul|
          h_id = vul['id']
          c_dt = vul['create_datetime']
          u_dt = vul['update_datetime']
          log_debug("htmlid_date: info: #{h_id} #{c_dt} #{u_dt}")
          if h_id and (c_dt or u_dt) then
            h_id = h_id.to_i
            if @htmlid_date_htmlids.include?(h_id) then
              log_debug(" already processed: skip")
              next
            end
            @htmlid_date_htmlids << h_id
            SidfmdHtmlidDate.transaction do
              r = SidfmdHtmlidDate.lock.find_by_htmlid(h_id)
              if r then
                log_debug("htmlid_date: found: #{r.inspect}")
                p_c_dt = nil
                p_u_dt = nil
                p_c_dt = c_dt.to_datetime if c_dt
                p_u_dt = u_dt.to_datetime if u_dt
                updates = {}
                if r[:create_datetime].nil? or (c_dt and r[:create_datetime] and r[:create_datetime] < p_c_dt) then
                  updates[:create_datetime] = c_dt
                end
                if r[:update_datetime].nil? or (u_dt and r[:update_datetime] and r[:update_datetime] < p_u_dt) then
                  updates[:update_datetime] = u_dt
                end
                if updates.size > 0 then
                  r.update_attributes(updates)
                  log_debug("htmlid_date: update: #{r.inspect}")
                end
              else
                r = SidfmdHtmlidDate.create(:htmlid => h_id, :create_datetime => c_dt, :update_datetime => u_dt)
                log_debug("htmlid_date: create: #{r.inspect}")
              end
            end
          end
        end
        #
        ActiveRecord::Base.transaction do
          ActiveRecord::Base.transaction(:requires_new => true) do
            # 古いSidfmdQueue*があれば削除する
            old_vuls = queue.sidfmd_queue_vuls
            old_vuls.each do |old_vul|
              old_vul.sidfmd_queue_vul_affprods.delete_all
              old_vul.sidfmd_queue_vul_affvers.delete_all
              old_vul.sidfmd_queue_vul_affpkgs.delete_all
              old_vul.sidfmd_queue_vul_cves.delete_all
              old_vul.sidfmd_queue_vul_sris.delete_all
            end
            old_vuls.delete_all
            # SidfmdQueue*に照合した結果を保存
            queue_vuls = []
            vuls.each do |vul|
              queue_vuls << issues_update_main_vul(queue, vul)
            end
            SidfmdQueueVul.import(queue_vuls, validate: false, recursive: true) if queue_vuls.any?
            unless queue.update_attribute(:done, true) then
              raise ActiveRecord::Rollback
            end
          end
          ActiveRecord::Base.transaction(:requires_new => true) do
            # SidfmdQueue*に保存した脆弱性情報と、既に登録されている脆弱性
            # (SidfmdIssue*)とを照合して、適切な処理を行う
            issues = host.sidfmd_issues
            if queue then
              # 以下の行では動かないので注意。
              #queue_vuls = queue.sidfmd_queue_vuls
              queue_vuls = SidfmdQueueVul.where(:sidfmd_queue_id => queue.id)
              if queue_vuls and queue_vuls.count > 0 then
                queue_vuls.each do |queue_vul|
                  issues_update_main_queue_vul(host, issues, queue_vul)
                end
              end
              ### 既に登録済のissueが今回検出されなかった場合。
              # 差分更新の場合はskip
              if flag_diff then
                issues_update_main_collation(host, issues, queue_vuls)
              end
              unless queue.update_attribute(:checked, true) then
                raise ActiveRecord::Rollback
              end
            end
            host.update_attribute :last_checked_at, Time.current
            host.update_attribute :last_checked_hash, new_hash
          end
        end
      end
    end

    def issues_update_main_vul(queue, vul)
      ret = SidfmdQueueVul.new
      ret.html_id = vul['id']
      ret.sidfmd_queue_id = queue.id
      ret.title = vul['title']
      ret.area_id = vul['area_id']
      ret.risk_id = vul['risk_id']
      ret.short_description = vul['short_description'] if SidfmdQueueVul.column_names.include?('short_description')
      # affprods
      affprods = vul['affect_products']
      if affprods then
        affprods.each do |affprod|
          p = affprod['product']
          ret.sidfmd_queue_vul_affprods.build(product_id: p['id'], name: p['name'], create_datetime: p['create_datetime'], update_datetime: p['update_datetime'])
        end
      end
      # affvers
      affvers = vul['affect_product_versions']
      if affvers then
        affvers.each do |affver|
          v = affver['product_version']
          ret.sidfmd_queue_vul_affvers.build(product_version_id: v['id'], product_id: v['product_id'], fullname: v['fullname'], version: v['version'], create_datetime: v['create_datetime'], update_datetime: v['update_datetime'])
        end
      end
      # affpkgs
      affpkgs = vul['affect_packages']
      if affpkgs then
        affpkgs.each do |affpkg|
          ret.sidfmd_queue_vul_affpkgs.build(name: affpkg['name'], version: affpkg['version'], release: affpkg['release'], arch: affpkg['arch'])
        end
      end
      # cves
      cves = vul['cves']
      if cves then
        cves.each do |cve|
          opts = {}
          opts['cve_id'] = cve['cve_id']
          cvss2 = cve['cvss2']
          if cvss2 then
            opts['c2_vector'] = cvss2['vector']
            opts['c2_base_score'] = cvss2['base_score']
            opts['c2_create_datetime'] = cvss2['create_datetime']
            opts['c2_update_datetime'] = cvss2['update_datetime']
            opts['c2_exploitable'] = cvss2['exploitable']
          end
          cvss3 = cve['cvss3']
          if cvss3 then
            opts['c3_vector'] = cvss3['vector']
            opts['c3_base_score'] = cvss3['base_score']
            opts['c3_create_datetime'] = cvss3['create_datetime']
            opts['c3_update_datetime'] = cvss3['update_datetime']
            opts['c3_exploitable'] = cvss3['exploitable']
          end
          ret.sidfmd_queue_vul_cves.build(opts)
        end
      end

      # sris
      sris = vul['sris']
      if sris then
        sris.each do |sri|
          ret.sidfmd_queue_vul_sris.build(category_id: sri['category_id'], risk_id: sri['risk_id'], memo: sri['memo'])
        end
      end

      ret
    end

    def issues_update_main_queue_vul(host, issues, queue_vul)
      log_debug("issues_update_main_queue_vul: #{host.hostname} #{queue_vul.html_id}")
      exist_issue = issues.where(:html_id => queue_vul.html_id)
      if exist_issue.count == 0 then
        log_debug(" not exist")
        ### 対象ホストに登録されていないhtml_id(新規登録)
        opts_o_issue = {
          :tracker_id => VM_TRACKER.id,
          :project_id => VM_PROJECT.id,
          :subject => "[#{host.hostname}]" + queue_vul.title,
          :description => "新規脆弱性登録",
          :author_id => ADMIN_USER.id,
          :priority_id => DEFAULT_PRIORITY.id,
          :status_id => SidfmdIssue::STATUS_NOTCONFIRMED,
        }
        o_issue = Issue.create(opts_o_issue)
        log_debug(" not exist - pass 1")
        opts_l_issue = JSON.parse(queue_vul.to_json)
        opts_l_issue.delete('id')
        opts_l_issue.delete('sidfmd_queue_id')
        opts_l_issue['issue_id'] = o_issue.id
        opts_l_issue['sidfmd_host_id'] = host.id
        @rule_vulsync[:add_htmlids] << { :html_id => opts_l_issue['html_id'], :sidfmd_host_id => host.id }
        l_issue = SidfmdIssue.create(opts_l_issue)
        queue_vul_cves = queue_vul.sidfmd_queue_vul_cves
        if queue_vul_cves.count > 0 then
          log_debug(" not exist - pass 2")
          temp_data = []
          queue_vul_cves.each do |o_cve|
            temp_data << {
              sidfmd_issue_id: l_issue.id,
              cve_id: o_cve.cve_id,
              c2_vector: o_cve.c2_vector,
              c2_base_score: o_cve.c2_base_score,
              c2_create_datetime: o_cve.c2_create_datetime,
              c2_update_datetime: o_cve.c2_update_datetime,
              c2_exploitable: o_cve.c2_exploitable,
              c3_vector: o_cve.c3_vector,
              c3_base_score: o_cve.c3_base_score,
              c3_create_datetime: o_cve.c3_create_datetime,
              c3_update_datetime: o_cve.c3_update_datetime,
              c3_exploitable: o_cve.c3_exploitable,
            }
          end
          SidfmdIssueCve.import(temp_data, validate: false) if temp_data.any?
        end
        queue_vul_sris = queue_vul.sidfmd_queue_vul_sris
        if queue_vul_sris.count > 0 then
          log_debug(" not exist - pass 3")
          temp_data = []
          queue_vul_sris.each do |o_sri|
            temp_data << {
              sidfmd_issue_id: l_issue.id,
              category_id: o_sri.category_id,
              risk_id: o_sri.risk_id,
              memo: o_sri.memo,
            }
          end
          SidfmdIssueSri.import(temp_data, validate: false) if temp_data.any?
        end
        queue_vul_affprods = queue_vul.sidfmd_queue_vul_affprods
        if queue_vul_affprods.count > 0 then
          log_debug(" not exist - pass 4")
          temp_data = []
          queue_vul_affprods.each do |o_affprod|
            temp_data << {
              sidfmd_issue_id: l_issue.id,
              product_id: o_affprod.product_id,
              name: o_affprod.name,
              create_datetime: o_affprod.create_datetime,
              update_datetime: o_affprod.update_datetime,
            }
          end
          SidfmdIssueAffprod.import(temp_data, validate: false) if temp_data.any?
        end
        queue_vul_affvers = queue_vul.sidfmd_queue_vul_affvers
        if queue_vul_affvers.count > 0 then 
          log_debug(" not exist - pass 5")
          temp_data = []
          queue_vul_affvers.each do |o_affver|
            temp_data << {
              sidfmd_issue_id: l_issue.id,
              product_version_id: o_affver.product_version_id,
              product_id: o_affver.product_id,
              fullname: o_affver.fullname,
              version: o_affver.version,
              create_datetime: o_affver.create_datetime,
              update_datetime: o_affver.update_datetime,
            }
          end
          SidfmdIssueAffver.import(temp_data, validate: false) if temp_data.any?
        end
        queue_vul_affpkgs = queue_vul.sidfmd_queue_vul_affpkgs
        if queue_vul_affpkgs.count > 0 then
          log_debug(" not exist - pass 6")
          temp_data = []
          queue_vul_affpkgs.each do |o_affpkg|
            temp_data << {
              sidfmd_issue_id: l_issue.id,
              name: o_affpkg.name,
              version: o_affpkg.version,
              release: o_affpkg.release,
              arch: o_affpkg.arch,
            }
          end
          SidfmdIssueAffpkg.import(temp_data, validate: false) if temp_data.any?
        end
        system_category = host.system_category
        if system_category then
          log_debug(" not exist - pass 7")
          sri = l_issue.get_sri
          if sri then
            position_name = 'lowest'
            case sri.risk_id
            when 4 then 'high2'
            when 3 then 'high3'
            when 2 then 'default'
            when 1 then 'lowest'
            end
            unless o_issue.update_attribute(:priority_id, IssuePriority.find_by_position_name(position_name).id) then
              raise ActiveRecord::Rollback
            end
          end
        end
      else
        log_debug(" exist")
        ### 対象ホストに登録済のhtml_id(必要なら更新)
        exist_issue = exist_issue.first
        o_issue = exist_issue.issue
        # 全体のアップデート確認フラグ。
        update_flag = false
        change_prodver_flag = false
        change_cve_flag = false
        affect_packages_count = 0
        # exist_issueとissueの更新
        if exist_issue.title != queue_vul.title then
          log_debug(" exist - pass 1")
          update_flag = true
          log_info("vm_daemon/title change: from #{exist_issue.title} to #{queue_vul.title}")
          exist_issue.update_attribute :title, queue_vul.title
          o_issue.update_attribute :subject, "[#{host.hostname}]" + queue_vul.title
        end
        if exist_issue.risk_id != queue_vul.risk_id then
          log_debug(" exist - pass 2")
          update_flag = true
          log_info("vm_daemon/risk_id change: from #{exist_issue.risk_id} to #{queue_vul.risk_id}")
          exist_issue.update_attribute :risk_id, queue_vul.risk_id
        end
        if exist_issue.area_id != queue_vul.area_id then
          log_debug(" exist - pass 3")
          update_flag = true
          log_info("vm_daemon/area_id change: from #{exist_issue.area_id} to #{queue_vul.area_id}")
          exist_issue.update_attribute :area_id, queue_vul.area_id
        end
        if SidfmdQueueVul.column_names.include?('short_description') and exist_issue.short_description != queue_vul.short_description then
          log_debug(" exist - pass 4")
          update_flag = true
          log_info("vm_daemon/short_description change: from #{exist_issue.short_description} to #{queue_vul.short_description}")
          exist_issue.update_attribute :short_description, queue_vul.short_description
        end
        # affect_product_versions
        exist_verids = exist_issue.sidfmd_issue_affvers.map(&:product_version_id).uniq.compact
        queuevul_verids = queue_vul.sidfmd_queue_vul_affvers.map(&:product_version_id).uniq.compact
        unless (exist_verids - queuevul_verids).size == 0 and (queuevul_verids - exist_verids).size == 0 then
          log_debug(" exist - pass 5")
          log_info("vm_daemon/affvers change: from #{exist_verids.to_json} to #{queuevul_verids.to_json}")
          exist_issue.sidfmd_issue_affvers.delete_all
          temp_data = []
          queue_vul.sidfmd_queue_vul_affvers.each do |queue_affver|
            temp_data << {
              sidfmd_issue_id: exist_issue.id,
              product_version_id: queue_affver.product_version_id,
              product_id: queue_affver.product_id,
              fullname: queue_affver.fullname,
              version: queue_affver.version,
              create_datetime: queue_affver.create_datetime,
              update_datetime: queue_affver.update_datetime,
            }
          end
          SidfmdIssueAffver.import(temp_data, validate: false) if temp_data.any?
          change_prodver_flag = true
          update_flag = true
        end
        # affect_products
        exist_prodids = exist_issue.sidfmd_issue_affprods.map(&:product_id).uniq.compact
        queuevul_prodids = queue_vul.sidfmd_queue_vul_affprods.map(&:product_id).uniq.compact
        unless (exist_prodids - queuevul_prodids).size == 0 and (queuevul_prodids - exist_prodids).size == 0 then
          log_debug(" exist - pass 6")
          log_info("vm_daemon/affprods change: from #{exist_prodids.to_json} to #{queuevul_prodids.to_json}")
          exist_issue.sidfmd_issue_affprods.delete_all
          temp_data = []
          queue_vul.sidfmd_queue_vul_affprods.each do |queue_affprod|
            temp_data << {
              sidfmd_issue_id: exist_issue.id,
              product_id: queue_affprod.product_id,
              name: queue_affprod.name,
              create_datetime: queue_affprod.create_datetime,
              update_datetime: queue_affprod.update_datetime,
            }
          end
          SidfmdIssueAffprod.import(temp_data, validate: false) if temp_data.any?
          change_prodver_flag = true
          update_flag = true
        end
        # affect_packages
        exist_pkgs = exist_issue.sidfmd_issue_affpkgs.collect {|x| "#{x.name}/#{x.version}/#{x.release}/#{x.arch}" }.uniq
        queuevul_pkgs = queue_vul.sidfmd_queue_vul_affpkgs.collect {|x| "#{x.name}/#{x.version}/#{x.release}/#{x.arch}" }.uniq
        unless (exist_pkgs - queuevul_pkgs).size == 0 and (queuevul_pkgs - exist_pkgs).size == 0 then
          log_debug(" exist - pass 7")
          log_info("vm_daemon/affpkgs change: from #{exist_pkgs.to_json} to #{queuevul_pkgs.to_json}")
          exist_issue.sidfmd_issue_affpkgs.delete_all
          temp_data = []
          queue_vul.sidfmd_queue_vul_affpkgs.each do |queue_affpkg|
            temp_data << {
              sidfmd_issue_id: exist_issue.id,
              name: queue_affpkg.name,
              version: queue_affpkg.version,
              release: queue_affpkg.release,
              arch: queue_affpkg.arch,
            }
          end
          SidfmdIssueAffpkg.import(temp_data, validate: false) if temp_data.any?
          affect_packages_count = exist_issue.sidfmd_issue_affpkgs.count
          update_flag = true
        end
        # cves
        exist_cves = exist_issue.sidfmd_issue_cves.collect {|x| "#{x.cve_id}-#{x.c2_base_score}-#{x.c2_exploitable}-#{x.c3_base_score}-#{x.c3_exploitable}" }.uniq
        queuevul_cves = queue_vul.sidfmd_queue_vul_cves.collect {|x| "#{x.cve_id}-#{x.c2_base_score}-#{x.c2_exploitable}-#{x.c3_base_score}-#{x.c3_exploitable}" }.uniq
        unless (exist_cves - queuevul_cves).size == 0 and (queuevul_cves - exist_cves).size == 0 then
          log_debug(" exist - pass 8")
          log_info("vm_daemon/cve change: from #{exist_cves.to_json} to #{queuevul_cves.to_json}")
          exist_issue.sidfmd_issue_cves.delete_all
          temp_data = []
          queue_vul.sidfmd_queue_vul_cves.each do |o_cve|
            temp_data << {
              sidfmd_issue_id: exist_issue.id,
              cve_id: o_cve.cve_id,
              c2_vector: o_cve.c2_vector,
              c2_base_score: o_cve.c2_base_score,
              c2_create_datetime: o_cve.c2_create_datetime,
              c2_update_datetime: o_cve.c2_update_datetime,
              c2_exploitable: o_cve.c2_exploitable,
              c3_vector: o_cve.c3_vector,
              c3_base_score: o_cve.c3_base_score,
              c3_create_datetime: o_cve.c3_create_datetime,
              c3_update_datetime: o_cve.c3_update_datetime,
              c3_exploitable: o_cve.c3_exploitable,
            }
          end
          SidfmdIssueCve.import(temp_data, validate: false) if temp_data.any?
          change_cve_flag = true
          update_flag = true
        end
        # sris
        exist_sris = exist_issue.sidfmd_issue_sris.collect {|x| "#{x.category_id}/#{x.risk_id}/#{x.memo}" }.uniq
        queuevul_sris = queue_vul.sidfmd_queue_vul_sris.collect {|x| "#{x.category_id}/#{x.risk_id}/#{x.memo}" }.uniq
        unless (exist_sris - queuevul_sris).size === 0 and (queuevul_sris - exist_sris).size == 0 then
          log_debug(" exist - pass 9")
          log_info("vm_daemon/sri change: from #{exist_sris.to_json} to #{queuevul_sris.to_json}")
          exist_issue.sidfmd_issue_sris.delete_all
          temp_data = []
          queue_vul.sidfmd_queue_vul_sris.each do |o_sri|
            temp_data << {
              sidfmd_issue_id: exist_issue.id,
              category_id: o_sri.category_id,
              risk_id: o_sri.risk_id,
              memo: o_sri.memo,
            }
          end
          SidfmdIssueSri.import(temp_data, validate: false) if temp_data.any?
          update_flag = true
        end
        # 優先度の更新
        if o_issue.priority_id != DEFAULT_PRIORITY.id then
          log_debug(" exist - pass 10")
          system_category = host.system_category
          if system_category then
            sri = exist_issue.get_sri
            if sri then
              position_name = 'lowest'
              case sri.risk_id
              when 4 then 'high2'
              when 3 then 'high3'
              when 2 then 'default'
              when 1 then 'lowest'
              end
              unless o_issue.update_attribute(:priority_id, IssuePriority.find_by_position_name(position_name).id) then
                log_debug(" exist - pass 11")
                raise ActiveRecord::Rollback
              end
            end
          end
        end
        # cveが変更され、かつ、影響無のステータスを持つ場合
        check_cve = (change_cve_flag and SidfmdIssue::STATUS_NOTAFFECT == o_issue.status_id)
        # 影響のあるプロダクト・バージョンが変更され、かつ、影響のあるパッケージは無い場合
        check_prodver = (change_prodver_flag and affect_packages_count == 0 and SidfmdIssue::STATUS_FINISHED.include?(o_issue.status_id))
        ### FIXME: revert #24264
        #if check_cve then
        #  log_debug(" exist - pass 12a")
        #  log_debug("  check_prodver=#{check_prodver}")
        #  log_debug("  check_cve=#{check_cve}")
        #  log_info("vm_daemon/append journal(cve)")
        #  # ステータスを未対策に変更
        #  change_status_not_confirmed_cve(exist_issue)
        #elsif check_prodver then
        if check_prodver then
          log_debug(" exist - pass 12b")
          log_debug("  check_prodver=#{check_prodver}")
          log_debug("  check_cve=#{check_cve}")
          log_info("vm_daemon/append journal(prodver)")
          # ステータスを未対策に変更
          change_status_not_confirmed(exist_issue)
        end
        if update_flag then
          log_debug(" exist - pass 13")
          log_info("vm_daemon/update issues.updated_on")
          o_issue.update_attribute :updated_on, Time.current
          @rule_vulsync[:modify_htmlids] << { :html_id => exist_issue[:html_id], :sidfmd_host_id => exist_issue[:sidfmd_host_id] }
        end
      end
    end

    # queue_vulsとissuesの照合
    # 既に登録済のissueが今回検出されなかった場合の処理。
    def issues_update_main_collation(host, issues, queue_vuls)
      # html_id > 0は、Windows Agentの部分を排除するため。Windows Agentの部分に関しては
      # 別実装となる。
      removed_issues = issues
        .joins(:issue)
        .where('sidfmd_issues.html_id > 0')
        .where('sidfmd_issues.html_id NOT BETWEEN ? AND ?', SidfmdLocalVul::MIN_HTMLID, SidfmdLocalVul::MAX_HTMLID)
        .where.not(:html_id => queue_vuls.collect {|x| x.html_id }.uniq)
        .where(:issues => { :status_id => SidfmdIssue::STATUS_UNFINISHED })
      if removed_issues.count > 0 then
        removed_issues.each do |issue|
          # ステータスを対策済に変更
          change_status_fixed(issue)
        end
      end
    end

    # ローカル脆弱性の更新処理
    def issues_update_local(target_hosts)
      ret = SidfmdLocalVul.apply_vuls_to_hosts(target_hosts.collect {|x| x[:id] })
      add_sidfmd_issue_ids = ret[:add]
      modify_sidfmd_issue_ids = ret[:modify]
      delete_sidfmd_issue_ids = ret[:delete]
      add_htmlids = SidfmdIssue.select(:html_id).select(:sidfmd_host_id).group(:html_id, :sidfmd_host_id).where(:id => add_sidfmd_issue_ids).collect {|x| { :html_id => x[:html_id], :sidfmd_host_id => x[:sidfmd_host_id] } }
      modify_htmlids = SidfmdIssue.select(:html_id).select(:sidfmd_host_id).group(:html_id, :sidfmd_host_id).where(:id => modify_sidfmd_issue_ids).collect {|x| { :html_id => x[:html_id], :sidfmd_host_id => x[:sidfmd_host_id] } }
      delete_htmlids = SidfmdIssue.select(:html_id).select(:sidfmd_host_id).group(:html_id, :sidfmd_host_id).where(:id => delete_sidfmd_issue_ids).collect {|x| { :html_id => x[:html_id], :sidfmd_host_id => x[:sidfmd_host_id] } }
      @rule_vulsync[:add_htmlids] += add_htmlids
      @rule_vulsync[:modify_htmlids] += modify_htmlids
      @rule_vulsync[:remove_htmlids] += delete_htmlids
    end

    # agentで取得したdistを解析し、opts[:product_ids, :product_version_ids]に
    # 反映する
    def check_dist_pkgs(queue, opts)
      dist = queue.sidfmd_queue_dist
      if dist and dist.ident then
        case dist.ident
        when  'centos' then
          sidfmd_product = SidfmdProduct.find_by_name('CentOS')
          if dist.version then
            ver_splited = dist.version.split(/\./)
            if ver_splited.size > 0 then
              sidfmd_version = sidfmd_product.sidfmd_versions.find_by_version(ver_splited.first)
              if sidfmd_version then
                opts[:product_version_ids] << sidfmd_version.vid
              else
                # CentOS 8 の場合、Red Hat Enterprise Linux 8 での解決を試みる。
                r_sidfmd_product = SidfmdProduct.find_by_name('Red Hat Linux')
                sidfmd_version = r_sidfmd_product.sidfmd_versions.find_by_version("Enterprise Linux #{ver_splited.first}")
                if sidfmd_version then
                  opts[:product_version_ids] << sidfmd_version.vid
                else
                  # Red Hat Enterprise Linux でも適合しなければ、バージョンの認識を断念しプロダクトで登録。
                  opts[:product_ids] << sidfmd_product.pid
                end
              end
            end
          end
          opts[:packages] = queue.sidfmd_queue_pkgs.select(:pkg).collect {|x| x.pkg }
        when 'amzn' then
          sidfmd_product = SidfmdProduct.find_by_name('Amazon Linux AMI')
          sidfmd_version = sidfmd_product.sidfmd_versions.find_by_version(dist.version)
          if sidfmd_version then
            opts[:product_version_ids] << sidfmd_version.vid
          else
            opts[:product_ids] << sidfmd_product.pid
          end
          opts[:packages] = queue.sidfmd_queue_pkgs.select(:pkg).collect {|x| x.pkg }
        when 'rhel' then
          sidfmd_product = SidfmdProduct.find_by_name('Red Hat Linux')
          if dist.version then
            ver_splited = dist.version.split(/\./)
            if ver_splited.size > 0 then
              # Enterprise Linux Server 6/7
              sidfmd_version = sidfmd_product.sidfmd_versions.find_by_version("Enterprise Linux Server #{ver_splited.first}")
              if sidfmd_version then
                opts[:product_version_ids] << sidfmd_version.vid
              else
                # Enterprise Linux 5/8/9
                sidfmd_version = sidfmd_product.sidfmd_versions.find_by_version("Enterprise Linux #{ver_splited.first}")
                if sidfmd_version then
                  opts[:product_version_ids] << sidfmd_version.vid
                else
                  opts[:product_ids] << sidfmd_product.pid
                end
              end
            end
          end
          opts[:packages] = queue.sidfmd_queue_pkgs.select(:pkg).collect {|x| x.pkg }
        when 'ubuntu' then
          sidfmd_product = SidfmdProduct.find_by_name('Ubuntu')
          sidfmd_version = sidfmd_product.sidfmd_versions.find_by_version(dist.version + ' LTS')
          if sidfmd_version then
            opts[:product_version_ids] << sidfmd_version.vid
          else
            opts[:product_ids] << sidfmd_product.pid
          end
          opts[:packages] = queue.sidfmd_queue_pkgs.select(:pkg).collect {|x| x.pkg }
        when 'debian' then
          sidfmd_product = SidfmdProduct.find_by_name('Debian GNU/Linux')
          sidfmd_version = sidfmd_product.sidfmd_versions.find_by_version("#{dist.version}.0")
          if sidfmd_version then
            opts[:product_version_ids] << sidfmd_version.vid
          else
            opts[:product_ids] << sidfmd_product.pid
          end
          opts[:packages] = queue.sidfmd_queue_pkgs.select(:pkg).collect {|x| x.pkg }
        when 'almalinux' then
          sidfmd_product = SidfmdProduct.find_by_name('AlmaLinux')
          if dist.version then
            ver_splited = dist.version.split(/\./)
            if ver_splited.size > 0 then
              sidfmd_version = sidfmd_product.sidfmd_versions.find_by_version("#{ver_splited.first}")
              if sidfmd_version then
                opts[:product_version_ids] << sidfmd_version.vid
              else
                opts[:product_ids] << sidfmd_product.pid
              end
            end
          end
          opts[:packages] = queue.sidfmd_queue_pkgs.pluck(:pkg)
        end
      end
    end

    # agentで取得したアプリケーション情報を解析し、opts[:product_ids, :product_version_ids]に
    # 反映する
    def check_applications(queue, opts)
      apps = queue.sidfmd_queue_apps
      disabled_apps = SidfmdHostDisabledApp.where(:sidfmd_host_id => queue.sidfmd_host_id).pluck(:name, :version, :path)
      apps = apps.select{|app|
        disabled_apps.find{|d_app| d_app[0] == app.name && d_app[1] == app.version &&  d_app[2] == app.path } == nil
      } 
      if apps and apps.count > 0 then
        apps.each do |app|
          case app.name
          when 'Apache Struts' then check_application_struts(app, opts)
          when 'Apache Tomcat' then check_application_tomcat(app, opts)
          when 'PHP' then check_application_php(app, opts)
          when 'ruby' then check_application_ruby(app, opts)
          when 'Oracle Java' then check_application_jdk(app, opts)
          when 'Movable Type' then check_application_mt(app, opts)
          when 'WordPress' then check_application_wp(app, opts)
          when 'Joomla!' then check_application_joomla(app, opts)
          end
        end
      end
    end

    # Apache Struts
    def check_application_struts(app, opts)
      product = SidfmdProduct.find_by_name('Apache Struts')
      if product then
        version = product.sidfmd_versions.find_by_version(app.version)
        if version then
          opts[:product_version_ids] << version.vid
        else
          opts[:product_ids] << product.pid
        end
      end
    end

    # Apache Tomcat
    def check_application_tomcat(app, opts)
      product = SidfmdProduct.find_by_name('Tomcat')
      if product then
        version = product.sidfmd_versions.find_by_version(app.version)
        if version then
          opts[:product_version_ids] << version.vid
        else
          opts[:product_ids] << product.pid
        end
      end
    end

    # PHP
    def check_application_php(app, opts)
      product = SidfmdProduct.find_by_name('PHP')
      if product then
        version = product.sidfmd_versions.find_by_version(app.version)
        if version then
          opts[:product_version_ids] << version.vid
        else
          opts[:product_ids] << product.pid
        end
      end
    end

    # Ruby
    def check_application_ruby(app, opts)
      product = SidfmdProduct.find_by_name('Ruby')
      if product then
        version = product.sidfmd_versions.find_by_version(app.version)
        unless version then
          # 2.5.1p57 のようにパッチレベルがつくので、それを除去
          if app.version =~ /(\d+\.\d+\.\d+)/ then
            version = product.sidfmd_versions.find_by_version(Regexp.last_match(1))
          end
        end
        if version then
          opts[:product_version_ids] << version.vid
        else
          opts[:product_ids] << product.pid
        end
      end
    end

    # Oracke JDK
    def check_application_jdk(app, opts)
      product_jdk = SidfmdProduct.find_by_name('Oracle Java SE JDK')
      product_sdk = SidfmdProduct.find_by_name('Oracle Java SE SDK')
      version = nil
      if app.path =~ /jdk/i and app.path !~ /jre/i then 
        if product_jdk then
          version = product_jdk.sidfmd_versions.find_by_version(app.version)
        end
        unless version then
          if product_sdk then
            version = product_sdk.sidfmd_versions.find_by_version(app.version)
          end
        end
      end
      if version then
        opts[:product_version_ids] << version.vid
      else
        opts[:product_ids] << product_jdk.pid
      end
    end

    # Movable Type
    def check_application_mt(app, opts)
      product = SidfmdProduct.find_by_name('Movable Type')
      if product then
        if /\A(\d+)\.\d+\.\d+\z/ =~ app.version then
          version = product.sidfmd_versions.find_by_version("#{Regexp.last_match(1)}.x")
          if version then
            opts[:product_version_ids] << version.vid
          else
            opts[:product_ids] << product.pid
          end
        else
          opts[:product_ids] << product.pid
        end
      end
    end

    # WordPress
    def check_application_wp(app, opts)
      product = SidfmdProduct.find_by_name('WordPress')
      if product then
        if /\A(\d+\.\d+)(\.\d+)?\z/ =~ app.version then
          version = product.sidfmd_versions.find_by_version("#{Regexp.last_match(1)}.x")
          if version then
            opts[:product_version_ids] << version.vid
          else
            opts[:product_ids] << product.pid
          end
        else
          opts[:product_ids] << product.pid
        end
      end
    end

    # Joomla!
    def check_application_joomla(app, opts)
      product = SidfmdProduct.find_by_name('Joomla!')
      if product then
        if /\A(\d+\.\d+)\.\d+\z/ =~ app.version then
          version = product.sidfmd_versions.find_by_version("#{Regexp.last_match(1)}.x")
          if version then
            opts[:product_version_ids] << version.vid
          else
            opts[:product_ids] << product.pid
          end
        else
          opts[:product_ids] << product.pid
        end
      end
    end

    # ホスト一覧を取得し返す
    def get_hosts_content
      unless @rule_contents[:hosts_content] then
        log_debug("get_hosts_content: new : #{Time.current}")
        @rule_contents[:hosts_content] = []
        hv = SidfmdHostView
          .order(:hostname)
          .select(:sidfmd_host_id)
          .select(:hostname)
          .select(:ext_name)
          .select(:display_name)
          .select(:last_checked_at)
          .select(:exploitable)
          .select(:max_sri)
          .select(:max_area_id)
          .select(:min_status_id)
          .select(:max_status_id)
          .select(:max_cvss_score)
          .select(:max_cvss_label)
          .select(:max_cvss_exploitable)
          .select(:warning_sri)
          .select(:warning_cvss)
          .select(:system_category_id)
          .select(:system_category_name)
          .select(:osname)
          .select("
ARRAY_TO_STRING(ARRAY(

SELECT
 CAST(sidfmd_hv_categories.sidfmd_category_id AS text) || '|&&&&&|' ||
 sidfmd_hv_categories.name
FROM sidfmd_hv_categories
WHERE
 sidfmd_hv_categories.sidfmd_host_view_id=sidfmd_host_views.id

),',&&&&&,') AS nosyscat")
          .select("
ARRAY_TO_STRING(ARRAY(

SELECT
 COALESCE(TO_CHAR(sidfmd_hv_unf_due_dates.d,'YYYY/MM/DD'),'')
FROM sidfmd_hv_unf_due_dates
WHERE
 sidfmd_hv_unf_due_dates.sidfmd_host_view_id=sidfmd_host_views.id

),',') AS udd")
          .select("
ARRAY_TO_STRING(ARRAY(

SELECT
 CAST(sidfmd_hv_cvsses.score AS text) || '|' ||
 sidfmd_hv_cvsses.label || '|' ||
 CAST(sidfmd_hv_cvsses.exploitable AS text)
FROM sidfmd_hv_cvsses
WHERE
 sidfmd_hv_cvsses.sidfmd_host_view_id=sidfmd_host_views.id

),',') AS scvss")
         .select("
ARRAY_TO_STRING(ARRAY(

SELECT
 sidfmd_hv_scsids.status_id
FROM sidfmd_hv_scsids
WHERE
 sidfmd_hv_scsids.sidfmd_host_view_id=sidfmd_host_views.id

),',') AS statusids")
         .select("
           (
             ARRAY(
               SELECT ARRAY_TO_STRING(ARRAY[sidfmd_ctag_id, html_id], ':')
               FROM sidfmd_hv_ctags
               WHERE sidfmd_hv_ctags.sidfmd_host_view_id=sidfmd_host_views.id
             )
           ) AS ctags")
        hv.each do |host|
          data = {}
          data[:sidfmd_host_id] = host[:sidfmd_host_id]
          data[:hostname] = host[:hostname]
          data[:ext_name] = host[:ext_name]
          data[:display_name] = host[:display_name]
          data[:last_checked_at] = host[:last_checked_at]
          data[:exploitable] = host[:exploitable]
          data[:max_sri] = host[:max_sri]
          data[:max_area_id] = host[:max_area_id]
          data[:min_status_id] = host[:min_status_id]
          data[:max_status_id] = host[:max_status_id]
          data[:max_cvss] = {
            :score => 0.0,
            :label => :notreg,
            :exploitable => host[:max_cvss_exploitable],
          }
          if host[:max_cvss_score] then
            data[:max_cvss][:score] = host[:max_cvss_score]
          end
          if host[:max_cvss_label] then
            data[:max_cvss][:label] = host[:max_cvss_label].to_sym
          end
          data[:warning] = {
            :sri => host[:warning_sri],
            :cvss => host[:warning_cvss],
          }
          data[:os] = {}
          if host[:osname] then
            data[:os][:osname] = host[:osname]
          end
          data[:system_category] = {
            :id => host[:system_category_id],
            :name => host[:system_category_name],
          }
          data[:no_system_categories] = []
          if host[:nosyscat] then
            host[:nosyscat].split(/,&&&&&,/, -1).each do |cat|
              s_cat = cat.split(/\|&&&&&\|/, -1)
              if s_cat.size >= 2 then
                data[:no_system_categories] << {
                  :id => s_cat[0].to_i,
                  :name => s_cat[1],
                }
              end
            end
          end
          data[:due_dates_unfinished] = []
          if host[:udd] then
            host[:udd].split(/,/, -1).each do |d|
              data[:due_dates_unfinished] << (d.size > 0 ? d : nil)
            end
          end
          data[:cvss] = []
          if host[:scvss] then
            host[:scvss].split(/,/, -1).each do |c|
              s_c = c.split(/\|/, -1)
              data[:cvss] << {
                :score => (s_c[0] ? s_c[0].to_f : 0.0),
                :label => (s_c[1] ? s_c[1].to_sym : :notreg),
                :exploitable => (s_c[2] ? ("true" == s_c[2].downcase) : false),
              }
            end
          end
          data[:status_ids] = []
          if host[:statusids] then
            host[:statusids].split(/,/, -1).each do |c|
              data[:status_ids] << c.to_i
            end
          end
          data[:ctags] = []
          if host[:ctags] then
            host[:ctags].each do |ctag|
              ctag = ctag.split(':')
              data[:ctags] << { :ctag_id => ctag[0].to_i, :html_id => ctag[1].to_i } # SidfmdHost#create_all_contentに合わせる
            end
          end
          # 現時点でruleで使用していないものは取得しない。
          # :tl_issue_ids, :os(:osname以外), :warning_orig, :not_fixed_warning, , :fixed_warning, :due_dates, :created_ons, :elapsed_created_ons, :elapsed_days, :sri_cvss_status_ids
          @rule_contents[:hosts_content] << data
        end
        log_debug("get_hosts_content: new(finished) : #{Time.current}")
      else
        log_debug("get_hosts_content: reuse")
      end
      @rule_contents[:hosts_content]
    end

    # 脆弱性一覧を取得し返す
    def get_vuls_content
      unless @rule_contents[:vuls_content] then
        log_debug("get_vuls_content: new : #{Time.current}")
        @rule_contents[:vuls_content] = []
        vv = SidfmdVulView
          .order(:html_id => :desc)
          .select(:html_id)
          .select(:title)
          .select(:short_description)
          .select(:exploitable)
          .select(:max_sri)
          .select(:min_status_id)
          .select(:min_created_on)
          .select(:min_created_on_display)
          .select(:softek_risk_id)
          .select(:softek_area_id)
          .select(:cvss_score)
          .select(:cvss_label)
          .select(:warning_sri)
          .select(:warning_cvss)
          .select(:fixed_warning_sri)
          .select(:fixed_warning_cvss)
          .select(:not_fixed_warning_sri)
          .select(:not_fixed_warning_cvss)
          .select(:min_elapsed_created_on)
          .select(:min_elapsed_created_on_display)
          .select("
ARRAY_TO_STRING(ARRAY(

SELECT
 COALESCE(TO_CHAR(sidfmd_vv_unf_due_dates.dt,'YYYY/MM/DD'),'')
FROM sidfmd_vv_unf_due_dates
WHERE
 sidfmd_vv_unf_due_dates.sidfmd_vul_view_id=sidfmd_vul_views.id

),',') AS udd")
    .select("
ARRAY_TO_STRING(ARRAY(

SELECT
 sidfmd_vv_cves.cve_id || '|' ||
 COALESCE(CAST(sidfmd_vv_cves.c2_base_score AS text),'-1.0') || '|' ||
 COALESCE(CAST(sidfmd_vv_cves.c3_base_score AS text),'-1.0') || '|' ||
 (CASE WHEN sidfmd_vv_cves.c2_av IS NOT NULL
       THEN
         'AV:' || sidfmd_vv_cves.c2_av || '/' ||
         'AC:' || sidfmd_vv_cves.c2_ac || '/' ||
         'Au:' || sidfmd_vv_cves.c2_au || '/' ||
         'C:'  || sidfmd_vv_cves.c2_c  || '/' ||
         'I:'  || sidfmd_vv_cves.c2_i  || '/' ||
         'A:'  || sidfmd_vv_cves.c2_a
       ELSE '' END) || '|' ||
 (CASE WHEN sidfmd_vv_cves.c3_av IS NOT NULL
       THEN
         'AV:' || sidfmd_vv_cves.c3_av || '/' ||
         'AC:' || sidfmd_vv_cves.c3_ac || '/' ||
         'PR:' || sidfmd_vv_cves.c3_pr || '/' ||
         'UI:' || sidfmd_vv_cves.c3_ui || '/' ||
         'S:'  || sidfmd_vv_cves.c3_s  || '/' ||
         'C:'  || sidfmd_vv_cves.c3_c  || '/' ||
         'I:'  || sidfmd_vv_cves.c3_i  || '/' ||
         'A:'  || sidfmd_vv_cves.c3_a
       ELSE '' END)
FROM sidfmd_vv_cves
WHERE
 sidfmd_vv_cves.sidfmd_vul_view_id=sidfmd_vul_views.id AND
 sidfmd_vv_cves.id=(
   SELECT id
   FROM sidfmd_vv_cves AS cve_table2
   WHERE sidfmd_vv_cves.sidfmd_vul_view_id=cve_table2.sidfmd_vul_view_id AND
         sidfmd_vv_cves.cve_id=cve_table2.cve_id
   ORDER BY
     CASE
     WHEN c3_update_datetime IS NOT NULL THEN
       100
     WHEN c2_update_datetime IS NOT NULL THEN
       200
     ELSE
       300
     END,
     c3_update_datetime DESC,
     c2_update_datetime DESC,
     id DESC
     LIMIT 1
 )

),',') AS cves")
          .select("
ARRAY_TO_STRING(ARRAY(

SELECT
 CAST(sidfmd_vv_hosts.sidfmd_host_id AS text) || ':&&&&&:' ||
 sidfmd_vv_hosts.hostname || ':&&&&&:' ||
 COALESCE(sidfmd_vv_hosts.ext_name,'') || ':&&&&&:' ||
 sidfmd_vv_hosts.display_name
FROM sidfmd_vv_hosts
WHERE
 sidfmd_vv_hosts.sidfmd_vul_view_id=sidfmd_vul_views.id

),',&&&&&,') AS hosts")
          .select("
ARRAY_TO_STRING(ARRAY(

SELECT
 CAST(sidfmd_vv_categories.sidfmd_category_id AS text) || '|&&&&&|' ||
 CAST(sidfmd_vv_categories.is_system AS text) || '|&&&&&|' ||
 sidfmd_vv_categories.name
FROM sidfmd_vv_categories
WHERE
 sidfmd_vv_categories.sidfmd_vul_view_id=sidfmd_vul_views.id

),',&&&&&,') AS categories")
          .select("
ARRAY_TO_STRING(ARRAY(

SELECT
 CAST(sidfmd_vv_scsids.sidfmd_host_id AS text) || '|' ||
 CAST(sidfmd_vv_scsids.status_id AS text) || '|' ||
 COALESCE(TO_CHAR(sidfmd_vv_scsids.due_date,'YYYY-MM-DD'),'') || '|' ||
 CAST(sidfmd_vv_scsids.created_on AS text) || '|' ||
 CAST(sidfmd_vv_scsids.sri AS text) || '|' ||
 COALESCE(CAST(sidfmd_vv_scsids.cvss AS text),'')
FROM sidfmd_vv_scsids
WHERE
 sidfmd_vv_scsids.sidfmd_vul_view_id=sidfmd_vul_views.id

),',') AS scsids")
          .select("
            (
              ARRAY(
                SELECT ARRAY_TO_STRING(ARRAY[sidfmd_ctag_id, sidfmd_host_id], ':')
                FROM sidfmd_vv_ctags
                WHERE sidfmd_vv_ctags.sidfmd_vul_view_id=sidfmd_vul_views.id
              )
            ) AS ctags")
        vv.each do |vul|
          data = {}
          data[:html_id] = vul[:html_id]
          data[:title] = vul[:title]
          data[:short_description] = vul[:short_description]
          data[:exploitable] = vul[:exploitable]
          data[:max_sri] = vul[:max_sri]
          data[:min_status_id] = vul[:min_status_id]
          data[:min_created_on] = {
            :value => vul[:min_created_on],
            :display => vul[:min_created_on_display],
            :elapsed_days => vul[:min_created_on] ? ((Time.current - vul[:min_created_on]) / 24.0 / 60.0 / 60.0).to_i : -1,
          }
          data[:min_elapsed_created_on] = {
            :value => vul[:min_elapsed_created_on],
            :display => (vul[:min_elapsed_created_on_display] || '-1').to_i,
          }
          data[:softek] = {
            :risk_id => vul[:softek_risk_id],
            :area_id => vul[:softek_area_id],
          }
          data[:cvss] = {
            :score => vul[:cvss_score],
            :label => vul[:cvss_label] ? vul[:cvss_label].to_sym : :notreg,
          }
          data[:warning] = {
            :sri => vul[:warning_sri],
            :cvss => vul[:warning_cvss],
          }
          data[:fixed_warning] = {
            :sri => vul[:fixed_warning_sri],
            :cvss => vul[:fixed_warning_cvss],
          }
          data[:not_fixed_warning] = {
            :sri => vul[:not_fixed_warning_sri],
            :cvss => vul[:not_fixed_warning_cvss],
          }
          data[:due_dates_unfinished] = []
          if vul[:udd] then
            vul[:udd].split(/,/, -1).each do |dt|
              data[:due_dates_unfinished] << (dt.size > 0 ? dt : nil)
            end
          end
          data[:cves] = []
          if vul[:cves] then
            vul[:cves].split(/,/, -1).each do |cve|
              s_cve = cve.split(/\|/, -1)
              if s_cve.size == 5 then
                data[:cves] << {
                  :cve_id => s_cve[0],
                  :c2_base_score => s_cve[1].to_f,
                  :c3_base_score => s_cve[2].to_f,
                  :c2_vector => s_cve[3],
                  :c3_vector => s_cve[4],
                }
              end
            end
          end
          data[:hosts] = []
          if vul[:hosts] then
            vul[:hosts].split(/,&&&&&,/, -1).each do |host|
              s_host = host.split(/:&&&&&:/, -1)
              if s_host.size == 4 then
                data[:hosts] << {
                  :id => s_host[0].to_i,
                  :hostname => s_host[1],
                  :ext_name => s_host[2].size > 0 ? s_host[2] : nil,
                  :display_name => s_host[3],
                }
              end
            end
          end
          data[:categories] = []
          if vul[:categories] then
            vul[:categories].split(/,&&&&&,/, -1).each do |category|
              s_cat = category.split(/\|&&&&&\|/, -1)
              if s_cat.size == 3 then
                data[:categories] << {
                  :id => s_cat[0].to_i,
                  :is_system => ("true" == s_cat[1]),
                  :name => s_cat[2],
                }
              end
            end
          end
          data[:sri_cvss_status_ids] = []
          if vul[:scsids] then
            vul[:scsids].split(/,/, -1).each do |scsid|
              s_scsid = scsid.split(/\|/, -1)
              if s_scsid.size == 6 then
                data[:sri_cvss_status_ids] << {
                  :sidfmd_host_id => s_scsid[0].to_i,
                  :status_id => s_scsid[1].to_i,
                  :due_date => (s_scsid[2].size > 0 ? s_scsid[2] : nil),
                  :created_on => (begin; Time.parse(s_scsid[3]); rescue ; nil ; end),
                  :sri => s_scsid[4].to_i,
                  :cvss => (s_scsid[5].size > 0 ? s_scsid[5].to_f : nil),
                }
              end
            end
            data[:status_ids] = data[:sri_cvss_status_ids].collect {|x| x[:status_id] }
          end
          data[:status_ids] = [] unless data.has_key?(:status_ids)
          data[:ctags] = []
          if vul[:ctags] then
            vul[:ctags].each do |ctag|
              ctag = ctag.split(':')
              data[:ctags] << { :ctag_id => ctag[0].to_i, :sidfmd_host_id => ctag[1].to_i } #SidfmdIssue#create_all_contentに合わせる
            end
          end
          # 現時点でruleで使用していないものは取得しない。
          # :created_ons, :elapsed_created_ons ] :not_fixed_hosts :warning_orig, :due_dates, :cvesのc2/c3_create/update_datetime
          @rule_contents[:vuls_content] << data
        end
        log_debug("get_vuls_content: new(finished) : #{Time.current}")
      else
        log_debug("get_vuls_content: reuse")
      end
      @rule_contents[:vuls_content]
    end

    # add/remove/modify_htmlidsに記載されているsidfmd_host_id以外の
    # sri_cvss_status_idsの値を除去し、その内容に従ってrowの内容を
    # 更新した結果を返す。
    # もしsri_cvss_status_ids.size == 0になったら、nilを返す。
    def get_vulsync_content_sub(htmlids, html_id, corresp_required)
      hostids = htmlids.find_all {|y| html_id == y[:html_id]}.collect {|y| y[:sidfmd_host_id] }.compact.uniq
      ret = @rule_contents[:vuls_content].find {|y| html_id == y[:html_id]}
      if ret then
        orig_size = ret[:sri_cvss_status_ids].size
        ret[:sri_cvss_status_ids].delete_if {|y|
          !hostids.include?(y[:sidfmd_host_id])
        }
        if ret[:sri_cvss_status_ids].size == 0 then
          ret = nil
        elsif orig_size != ret[:sri_cvss_status_ids].size then
          SidfmdIssue.update_value_by_scsids(ret, corresp_required)
        end
      end
      ret
    end

    # 脆弱性同期時に変更されたデータを取得し返す
    def get_vulsync_content
      get_vuls_content
      corresp_required = Sidfm.corresp_required
      # add - 新規脆弱性
      add_htmlids = @rule_vulsync[:add_htmlids].collect {|x| x[:html_id] }.uniq.compact
      @rule_contents[:vulsync_content][:add] = add_htmlids.collect {|x|
        get_vulsync_content_sub(@rule_vulsync[:add_htmlids], x, corresp_required)
      }.compact
      # remove - 自動ステータス変更により「対応済」「影響無」になった脆弱性
      remove_htmlids = @rule_vulsync[:remove_htmlids].collect {|x| x[:html_id] }.uniq.compact
      @rule_contents[:vulsync_content][:remove] = remove_htmlids.collect {|x|
        get_vulsync_content_sub(@rule_vulsync[:remove_htmlids], x, corresp_required)
      }.compact
      # modify - 脆弱性同期で、脆弱性情報に変更があったもの
      modify_htmlids = @rule_vulsync[:modify_htmlids].collect {|x| x[:html_id] }.uniq.compact
      @rule_contents[:vulsync_content][:modify] = modify_htmlids.collect {|x|
        get_vulsync_content_sub(@rule_vulsync[:modify_htmlids], x, corresp_required)
      }.compact
    end

    # ホストのタイムライン処理
    def host_timeline_update_sub(sidfmd_category_id, flg_fixed)
      sri = { :critical => 0, :high => 0, :medium => 0, :low => 0, :none => 0, :fixed => 0 }
      cvss = { :d4 => 0, :d3 => 0, :d2 => 0, :d1 => 0, :notreg => 0, :fixed => 0 }
      status = { :notconfirmed => 0, :ongoing => 0, :fixed => 0, :notaffect => 0 }
      host_views = SidfmdHostView.all
      if sidfmd_category_id then
        host_ids =
          SidfmdCategoriesHost.
          select(:sidfmd_host_id).
          joins(:sidfmd_category).
          joins(:sidfmd_host).
          where(:sidfmd_hosts => { :flg_delete => false }).
          where(:sidfmd_category_id => sidfmd_category_id).
          distinct.
          map(&:sidfmd_host_id).uniq
        host_views = host_views.user_host_view(false, host_ids)
      end
      host_views.each do |i|
        if flg_fixed then
          if SidfmdIssue::STATUS_UNFINISHED.include?(i[:min_status_id]) then
            case i[:max_sri]
            when 0 then sri[:none] += 1
            when 1 then sri[:low] += 1
            when 2 then sri[:medium] += 1
            when 3 then sri[:high] += 1
            when 4 then sri[:critical] += 1
            else sri[:none] += 1
            end
            if i[:max_cvss_label] then
              cvss[i[:max_cvss_label].to_sym] += 1
            else
              cvss[:d1] += 1
            end
          else
            sri[:fixed] += 1
            cvss[:fixed] += 1
          end
        else
          case i[:max_sri]
          when 0 then sri[:none] += 1
          when 1 then sri[:low] += 1
          when 2 then sri[:medium] += 1
          when 3 then sri[:high] += 1
          when 4 then sri[:critical] += 1
          else sri[:none] += 1
          end
          if i[:max_cvss_label] then
            cvss[i[:max_cvss_label].to_sym] += 1
          else
            cvss[:d1] += 1
          end
        end
        case i[:min_status_id]
        when SidfmdIssue::STATUS_NOTCONFIRMED then status[:notconfirmed] += 1
        when SidfmdIssue::STATUS_ONGOING then status[:ongoing] += 1
        when SidfmdIssue::STATUS_FIXED then status[:fixed] += 1
        when SidfmdIssue::STATUS_NOTAFFECT then status[:notaffect] += 1
        else status[:notaffect] += 1
        end
      end
      data = {
        :sidfmd_category_id => sidfmd_category_id,
        :dt => Time.current,
        :sri_critical => sri[:critical],
        :sri_high => sri[:high],
        :sri_medium => sri[:medium],
        :sri_low => sri[:low],
        :sri_none => sri[:none],
        :cvss_d4 => cvss[:d4],
        :cvss_d3 => cvss[:d3],
        :cvss_d2 => cvss[:d2],
        :cvss_d1 => cvss[:d1],
        :cvss_notreg => cvss[:notreg],
        :status_notconfirmed => status[:notconfirmed],
        :status_ongoing => status[:ongoing],
        :status_fixed => status[:fixed],
        :status_notaffect => status[:notaffect],
      }
      if flg_fixed then
        data[:sri_fixed] = sri[:fixed]
        data[:cvss_fixed] = cvss[:fixed]
      end
      data
    end

    def host_timeline_update
      flg_fixed = (SidfmdHostTimeline.column_names.include?('sri_fixed') && SidfmdHostTimeline.column_names.include?('cvss_fixed'))
      data = host_timeline_update_sub(nil, flg_fixed)
      begin
        SidfmdHostTimeline.create(data)
      rescue Exception => e
        log_error(e)
      end
      SidfmdCategory.all.each do |q|
        data = host_timeline_update_sub(q.id, flg_fixed)
        begin
          SidfmdHostTimeline.create(data)
        rescue Exception => e
          log_error(e)
        end
      end
    end

    # 脆弱性のタイムライン処理
    def vul_timeline_update_sub(sidfmd_category_id, flg_fixed)
      sri = { :critical => 0, :high => 0, :medium => 0, :low => 0, :none => 0, :fixed => 0 }
      cvss = { :d4 => 0, :d3 => 0, :d2 => 0, :d1 => 0, :notreg => 0, :fixed => 0 }
      status = { :notconfirmed => 0, :ongoing => 0, :fixed => 0, :notaffect => 0 }
      vul_views = SidfmdVulView.all
      host_ids = nil
      if sidfmd_category_id then
        host_ids =
          SidfmdCategoriesHost.
          select(:sidfmd_host_id).
          joins(:sidfmd_category).
          joins(:sidfmd_host).
          where(:sidfmd_hosts => { :flg_delete => false }).
          where(:sidfmd_category_id => sidfmd_category_id).
          distinct.
          map(&:sidfmd_host_id).uniq
        vul_views = vul_views.
          where(:id =>
                SidfmdVvScsid.
                select(:sidfmd_vul_view_id).
                where(:sidfmd_host_id => host_ids).
                distinct)
      end
      vul_views.each do |i|
        scsids = i.sidfmd_vv_scsids
        scsids = scsids.where(:sidfmd_host_id => host_ids) if sidfmd_category_id
        exist_scsids = (scsids.count > 0)
        min_status_id = sidfmd_category_id ? (exist_scsids ? scsids.minimum(:status_id) : 0) : i.min_status_id
        max_sri = sidfmd_category_id ? (exist_scsids ? scsids.maximum(:sri) : 0) : i.max_sri
        cvss_label = i.cvss_label.to_sym
        if sidfmd_category_id then
          if exist_scsids then
            r_cvss = scsids.order(:created_on => :desc).limit(1).first.cvss
            cvss_label = SidfmdIssueCve.labelling(r_cvss > 0.0 ? r_cvss : nil) if r_cvss
          end
        end
        if flg_fixed then
          if SidfmdIssue::STATUS_UNFINISHED.include?(min_status_id) then
            case max_sri
            when 0 then sri[:none] += 1
            when 1 then sri[:low] += 1
            when 2 then sri[:medium] += 1
            when 3 then sri[:high] += 1
            when 4 then sri[:critical] += 1
            else sri[:none] += 1
            end
            if cvss_label then
              cvss[cvss_label] += 1
            else
              cvss[:d1] += 1
            end
          else
            sri[:fixed] += 1
            cvss[:fixed] += 1
          end
        else
          case max_sri
          when 0 then sri[:none] += 1
          when 1 then sri[:low] += 1
          when 2 then sri[:medium] += 1
          when 3 then sri[:high] += 1
          when 4 then sri[:critical] += 1
          else sri[:none] += 1
          end
          if cvss_label then
            cvss[cvss_label] += 1
          else
            cvss[:d1] += 1
          end
        end
        case min_status_id
        when SidfmdIssue::STATUS_NOTCONFIRMED then status[:notconfirmed] += 1
        when SidfmdIssue::STATUS_ONGOING then status[:ongoing] += 1
        when SidfmdIssue::STATUS_FIXED then status[:fixed] += 1
        when SidfmdIssue::STATUS_NOTAFFECT then status[:notaffect] += 1
        else status[:notaffect] += 1
        end
      end
      data = {
        :sidfmd_category_id => sidfmd_category_id,
        :dt => Time.current,
        :sri_critical => sri[:critical],
        :sri_high => sri[:high],
        :sri_medium => sri[:medium],
        :sri_low => sri[:low],
        :sri_none => sri[:none],
        :cvss_d4 => cvss[:d4],
        :cvss_d3 => cvss[:d3],
        :cvss_d2 => cvss[:d2],
        :cvss_d1 => cvss[:d1],
        :cvss_notreg => cvss[:notreg],
        :status_notconfirmed => status[:notconfirmed],
        :status_ongoing => status[:ongoing],
        :status_fixed => status[:fixed],
        :status_notaffect => status[:notaffect],
      }
      if flg_fixed then
        data[:sri_fixed] = sri[:fixed]
        data[:cvss_fixed] = cvss[:fixed]
      end
      data
    end

    def vul_timeline_update
      flg_fixed = (SidfmdVulTimeline.column_names.include?('sri_fixed') && SidfmdVulTimeline.column_names.include?('cvss_fixed'))
      begin
        data = vul_timeline_update_sub(nil, flg_fixed)
        SidfmdVulTimeline.create(data)
      rescue Exception => e
        log_error(e)
      end
      SidfmdCategory.all.each do |q|
        data = vul_timeline_update_sub(q.id, flg_fixed)
        begin
          SidfmdVulTimeline.create(data)
        rescue Exception => e
          log_error(e)
        end
      end
    end

    # Sidfmd*Viewの更新
    def update_views
      log_debug("update_views:")
      log_debug(" params: see vm_rules_invoke")
      begin
        target_htmlids = []
        target_hostids = []
        [ :add_htmlids, :modify_htmlids, :remove_htmlids ].each do |key|
          target_htmlids |= @rule_vulsync[key].collect {|x| x[:html_id] }.uniq.compact
          target_hostids |= @rule_vulsync[key].collect {|x| x[:sidfmd_host_id] }.uniq.compact
        end
        target_htmlids.uniq!
        target_htmlids.compact!
        target_hostids.uniq!
        target_hostids.compact!
        
        log_debug("update_views: pass1 : #{Time.current}")
        result = SidfmdVulView.sync_data(target_htmlids)
        log_debug("SidfmdVulView.sync_data result:", result)
        log_debug("update_views: pass2 : #{Time.current}")
        result = SidfmdHostView.sync_data(target_hostids)
        log_debug("SidfmdHostView.sync_data result:", result)
        log_debug("update_views: pass3: #{Time.current}")
      rescue Exception => e
        log_error(e)
      end
      log_debug("update_views: end")
    end

    # VMRuleの処理
    def vm_rules_invoke
      log_debug("vm_rules_invoke:")
      log_debug("add_htmlids:", @rule_vulsync[:add_htmlids])
      log_debug("modify_htmlids:", @rule_vulsync[:modify_htmlids])
      log_debug("remove_htmlids:", @rule_vulsync[:remove_htmlids])
      # periodic
      periodic_rule_args = {}
      periodic_rule_ids = SIDfm::VMRule.invoke_rule_ids(:periodic, periodic_rule_args)
      if periodic_rule_ids.size > 0 then
        log_info("periodic_rule_invoke:", periodic_rule_ids)
        get_hosts_content
        get_vuls_content
        periodic_rule_args[:hosts_content] = @rule_contents[:hosts_content]
        periodic_rule_args[:vuls_content] = @rule_contents[:vuls_content]
        SIDfm::VMRule.invoke_rule_run(periodic_rule_ids, periodic_rule_args)
        log_info("periodic_rule_invoke: end")
      end
      # vulsync
      vulsync_rule_args = {}
      vulsync_rule_ids = SIDfm::VMRule.invoke_rule_ids(:vulsync, vulsync_rule_args)
      if vulsync_rule_ids.size > 0 then
        log_info("vulsync_rule_invoke:", vulsync_rule_ids)
        get_hosts_content
        get_vuls_content
        get_vulsync_content
        log_debug("sync_hostids:", @rule_vulsync[:sync_hostids])
        log_debug("content_add(size):", @rule_contents[:vulsync_content][:add].size)
        log_debug("content_modify(size):", @rule_contents[:vulsync_content][:modify].size)
        log_debug("content_remove(size):", @rule_contents[:vulsync_content][:remove].size)
        vulsync_rule_args[:sync_hostids] = @rule_vulsync[:sync_hostids]
        vulsync_rule_args[:prev_vuls_content] = @rule_vulsync[:prev_vuls_content]
        vulsync_rule_args[:content_add] = @rule_contents[:vulsync_content][:add]
        vulsync_rule_args[:content_modify] = @rule_contents[:vulsync_content][:modify]
        vulsync_rule_args[:content_remove] = @rule_contents[:vulsync_content][:remove]
        SIDfm::VMRule.invoke_rule_run(vulsync_rule_ids, vulsync_rule_args)
        log_info("vulsync_rule_invoke: end")
      end
      if periodic_rule_ids.size > 0 or vulsync_rule_ids.size > 0 then
        SIDfm::VMRule::Action.sync_data
        #SIDfm::VMRule::Action.sync_data2
      end
    end

    def execute
      target_hosts =
        SidfmdHost
        .enable_hosts
        .where('last_checked_at IS NULL OR last_checked_at < ?', Time.current.ago(Sidfm.vulsync_interval.hours))
        .order("COALESCE(last_checked_at,'epoch'::timestamp)")
        .limit(Sidfm.parallel_hosts)
      @rule_vulsync[:sync_hostids] = target_hosts.collect {|x| x.id }.uniq.compact
      log_debug("vm_daemon: target=#{target_hosts.collect {|x| x.id.to_s + '-' + x.hostname }.join(', ')}")
      products_update
      if target_hosts and target_hosts.count > 0 then
        areas_update
        risks_update
        issues_update(target_hosts)
        issues_update_local(target_hosts)
      end
      last_host_timeline = SidfmdHostTimeline.select(:dt).order(:dt => :desc).limit(1).first
      if !last_host_timeline or Time.current - last_host_timeline.dt > 24.0 * 60.0 * 60.0 then
        host_timeline_update
      end
      last_vul_timeline = SidfmdVulTimeline.select(:dt).order(:dt => :desc).limit(1).first
      if !last_vul_timeline or Time.current - last_vul_timeline.dt > 24.0 * 60.0 * 60.0 then
        vul_timeline_update
      end
      update_views
      vm_rules_invoke
    end

  end
end
